@@ -1,4 +1,4 @@
-/* $Id: softhsm-keyconv.cpp 5980 2011-12-22 14:21:18Z rb $ */
+/* $Id: softhsm-keyconv.cpp 7321 2013-09-23 12:05:15Z rb $ */
 
 /*
  * Copyright (c) 2009-2011 .SE (The Internet Infrastructure Foundation).
@@ -396,7 +396,7 @@ int to_pkcs8(char *in_path, char *out_path, char *file_pin) {
   // Save the the key to the disk
   switch(algorithm) {
     case DNS_KEYALG_ERROR:
-      fprintf(stderr, "Error: The algorithm was not given in the file.\n", algorithm);
+      fprintf(stderr, "Error: The algorithm %i was not given in the file.\n", algorithm);
       status = 1;
       break;
     case DNS_KEYALG_RSAMD5:
@@ -424,7 +424,6 @@ int to_pkcs8(char *in_path, char *out_path, char *file_pin) {
 int to_bind(char *in_path, char *file_pin, char *name, int ttl, int key_flag, char *algorithm_str) {
   int algorithm;
   Botan::Private_Key *priv_key;
-  char priv_out[MAX_LINE], pub_out[MAX_LINE];
   int status = 0;
 
   if(in_path == NULL) {
@@ -454,7 +453,7 @@ int to_bind(char *in_path, char *file_pin, char *name, int ttl, int key_flag, ch
   // Save keys to disk
   switch(algorithm) {
     case DNS_KEYALG_ERROR:
-      fprintf(stderr, "Error: The algorithm was not given in the file.\n", algorithm);
+      fprintf(stderr, "Error: The algorithm was not given in the file.\n");
       status = 1;
       break;
     case DNS_KEYALG_RSAMD5:
@@ -484,7 +483,6 @@ int to_bind(char *in_path, char *file_pin, char *name, int ttl, int key_flag, ch
 int save_rsa_pkcs8(char *out_path, char *file_pin, Botan::BigInt bigN, Botan::BigInt bigE,
                     Botan::BigInt bigD, Botan::BigInt bigP, Botan::BigInt bigQ) {
 
-  char buffer[MAX_LINE];
   Botan::Private_Key *priv_key = NULL;
   Botan::AutoSeeded_RNG *rng;
   int status = 0;
@@ -543,7 +541,6 @@ int save_rsa_pkcs8(char *out_path, char *file_pin, Botan::BigInt bigN, Botan::Bi
 int save_dsa_pkcs8(char *out_path, char *file_pin, Botan::BigInt bigDP, Botan::BigInt bigDQ, 
                     Botan::BigInt bigDG, Botan::BigInt bigDX) {
 
-  char buffer[MAX_LINE];
   Botan::Private_Key *priv_key = NULL;
   Botan::AutoSeeded_RNG *rng;
   int status = 0;
@@ -933,8 +930,8 @@ void print_big_int(FILE *file_pointer, const char *file_tag, Botan::BigInt big_i
 
 // Create RSA RDATA
 
-int create_rsa_rdata(unsigned char *rdata, int length, Botan::Private_Key *priv_key, int key_flag, int algorithm) {
-  unsigned int counter = 0, big_e_size, big_n_size;
+int create_rsa_rdata(unsigned char *rdata, size_t length, Botan::Private_Key *priv_key, int key_flag, int algorithm) {
+  size_t counter = 0, big_e_size, big_n_size;
   Botan::IF_Scheme_PrivateKey *if_key_priv;
   Botan::BigInt big_e;
   Botan::BigInt big_n;
@@ -995,8 +992,8 @@ int create_rsa_rdata(unsigned char *rdata, int length, Botan::Private_Key *priv_
 
 // Create DSA RDATA
 
-int create_dsa_rdata(unsigned char *rdata, int length, Botan::Private_Key *priv_key, int key_flag, int algorithm) {
-  unsigned int counter = 0, size, size_t;
+int create_dsa_rdata(unsigned char *rdata, size_t length, Botan::Private_Key *priv_key, int key_flag, int algorithm) {
+  size_t counter = 0, size, size_parameter;
   Botan::DL_Scheme_PrivateKey *dl_key_priv;
   Botan::BigInt big_q;
   Botan::BigInt big_p;
@@ -1015,10 +1012,10 @@ int create_dsa_rdata(unsigned char *rdata, int length, Botan::Private_Key *priv_
   big_g = dl_key_priv->group_g();
   big_y = dl_key_priv->get_y();
   size = big_g.bytes();
-  size_t = (size - 64) / 8;
+  size_parameter = (size - 64) / 8;
 
   // Check the value of T
-  if(size_t > 8) {
+  if(size_parameter > 8) {
     fprintf(stderr, "Error: create_dsa_rdata: No support for DSA T > 8.\n");
     return -1;
   }
@@ -1046,7 +1043,7 @@ int create_dsa_rdata(unsigned char *rdata, int length, Botan::Private_Key *priv_
   rdata[counter++] = (unsigned char)algorithm;
 
   // T
-  rdata[counter++] = (unsigned char)size_t;
+  rdata[counter++] = (unsigned char)size_parameter;
 
   // Q
   big_q.binary_encode((Botan::byte*)(rdata + counter));
@@ -1063,7 +1060,7 @@ int create_dsa_rdata(unsigned char *rdata, int length, Botan::Private_Key *priv_
   // Y
   big_y.binary_encode((Botan::byte*)(rdata + counter));
   counter += size;
-  
+
   return counter;
 }
 
@@ -1108,7 +1105,7 @@ int print_dnskey(FILE *file_pointer, char *name, int ttl, unsigned char *rdata,
 
 unsigned int keytag(unsigned char key[], unsigned int keysize) {
   unsigned long ac;     /* assumed to be 32 bits or larger */
-  int i;                /* loop index */
+  unsigned int i;       /* loop index */
 
   if(keysize < 4) {
     return 0;
