@@ -1,4 +1,4 @@
-/* $Id: mechanisms.cpp 6202 2012-03-07 10:43:44Z jakob $ */
+/* $Id: mechanisms.cpp 7276 2013-09-09 08:07:02Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -35,6 +35,56 @@
 #include "mechanisms.h"
 #include "log.h"
 
+// A list with the supported mechanisms
+static CK_MECHANISM_TYPE supportedMechanisms[] = {
+  CKM_RSA_PKCS_KEY_PAIR_GEN,
+  CKM_RSA_PKCS,
+  CKM_RSA_X_509,
+  CKM_MD5,
+  CKM_RIPEMD160,
+  CKM_SHA_1,
+  CKM_SHA256,
+  CKM_SHA384,
+  CKM_SHA512,
+  CKM_MD5_RSA_PKCS,
+  CKM_RIPEMD160_RSA_PKCS,
+  CKM_SHA1_RSA_PKCS,
+  CKM_SHA256_RSA_PKCS,
+  CKM_SHA384_RSA_PKCS,
+  CKM_SHA512_RSA_PKCS,
+  CKM_SHA1_RSA_PKCS_PSS,
+  CKM_SHA256_RSA_PKCS_PSS,
+  CKM_SHA384_RSA_PKCS_PSS,
+  CKM_SHA512_RSA_PKCS_PSS
+};
+
+CK_RV getMechanismList(CK_MECHANISM_TYPE_PTR pMechanismList, CK_ULONG_PTR pulCount) {
+  CK_ULONG listSize = sizeof(supportedMechanisms) / sizeof(supportedMechanisms[0]);
+
+  if(pMechanismList == NULL_PTR) {
+    *pulCount = listSize;
+
+    DEBUG_MSG("C_GetMechanismList", "OK, returning list length");
+    return CKR_OK;
+  }
+
+  if(*pulCount < listSize) {
+    *pulCount = listSize;
+
+    DEBUG_MSG("C_GetMechanismList", "Buffer to small");
+    return CKR_BUFFER_TOO_SMALL;
+  }
+
+  *pulCount = listSize;
+
+  for(unsigned int i = 0; i < listSize; i++) {
+    pMechanismList[i] = supportedMechanisms[i];
+  }
+
+  DEBUG_MSG("C_GetMechanismList", "OK, returning list");
+  return CKR_OK;
+}
+
 CK_RV getMechanismInfo(CK_MECHANISM_TYPE type, CK_MECHANISM_INFO_PTR pInfo) {
   CHECK_DEBUG_RETURN(pInfo == NULL_PTR, "C_GetMechanismInfo", "pInfo must not be a NULL_PTR",
                      CKR_ARGUMENTS_BAD);
@@ -73,6 +123,10 @@ CK_RV getMechanismInfo(CK_MECHANISM_TYPE type, CK_MECHANISM_INFO_PTR pInfo) {
     case CKM_SHA256_RSA_PKCS:
     case CKM_SHA384_RSA_PKCS:
     case CKM_SHA512_RSA_PKCS:
+    case CKM_SHA1_RSA_PKCS_PSS:
+    case CKM_SHA256_RSA_PKCS_PSS:
+    case CKM_SHA384_RSA_PKCS_PSS:
+    case CKM_SHA512_RSA_PKCS_PSS:
       pInfo->ulMinKeySize = 512;
       pInfo->ulMaxKeySize = 4096;
       pInfo->flags = CKF_SIGN | CKF_VERIFY;
