@@ -1,4 +1,4 @@
-/* $Id: tokenhandling.cpp 6202 2012-03-07 10:43:44Z jakob $ */
+/* $Id: tokenhandling.cpp 6270 2012-04-20 07:41:05Z jerry $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -79,7 +79,8 @@ static char sqlDeleteTrigger[] =
   "END;";
 
 static char sqlCreateIndexAttributes[] =
-  "CREATE INDEX idxObject ON Attributes (objectID, type);";
+  "CREATE INDEX idxObject ON Attributes (objectID, type);"
+  "CREATE INDEX idxTypeValue ON Attributes (type, value);";
 
 // Initialize a token
 
@@ -115,6 +116,8 @@ CK_RV softInitToken(SoftSlot *currentSlot, CK_UTF8CHAR_PTR pPin, CK_ULONG ulPinL
   EXEC_DB(db, "DROP TABLE IF EXISTS Objects");
   EXEC_DB(db, "DROP TABLE IF EXISTS Attributes");
   EXEC_DB(db, "DROP TRIGGER IF EXISTS deleteTrigger");
+  EXEC_DB(db, "DROP INDEX IF EXISTS idxObject");
+  EXEC_DB(db, "DROP INDEX IF EXISTS idxTypeValue");
   EXEC_DB(db, "VACUUM");
 
   // Add the structure
